% Project title: "Metabolic bistability and hysteresis in microbiome health
% and disease"
% Date updated: 3 Feb 2019
% By: Tahmineh Khazaei
% Before running the code call "initCobraToolbox" in MATLAB to load the
% COBRA toolbox


clear all;

    %Configure: COBRA Linear Programming Solver
    LPSolver = 'glpk';             
    changeCobraSolver(LPSolver, 'LP')

    %Loading the Models. 
        % Klebsiella pneumoniae
        model1 = readCbModel('KlebsiellaPublishedModel_2017.xml');
        fields_Kp = {'rules','genes','rxnGeneMat','grRules','subSystems','confidenceScores','rxnReferences','rxnECNumbers','rxnNotes'};
        model1 = rmfield(model1,fields_Kp);
        model1.description = 'oxygen_user';
        
        % Bacteroides thetaiotaomicron
        model2 = readCbModel('Published_Btheta_Thiele_2017.xml');
        fields_Bt = {'metFormulas'};
        model2 = rmfield(model2,fields_Bt);
        model2.description = 'dextran_user';
        
community.species = [model1 model2];
community.mets = {'EX_glc_D(e)', 'EX_dextran40(e)','EX_acetate','EX_malate', 'EX_propionate', 'EX_succinate'}; %metabolites tracked by DMMM
community.oxygen = {'EX_o2(e)'}; 
community.Xfeed = [0];				%biomass feed concentration
community.Fin = 0.04;               % units:L/hr; 40mL/hr is the flowrate
community.Fout = 0.04;              % units:L/hr; 40mL/hr is the flowrate


% Initial Conditions
initV = 0.2;                 %L
initX = [0.05 0.0015];       %g/L
initS = [0 0.06 0 0 0 0];    %mM
initO = 0.03;                %mmol
initialConditions = [initV initX initS initO];

% In order to store all FBA solutions
global vs_KP_glucose;
global vs_BT_glucose;
global vs_KP_dextran;
global vs_BT_dextran;
global vs_KP_acetate;
global vs_BT_acetate;
global mu_bt;
global mu_kp;

% OPTIONAL: If want to store all FBA solutions, uncomment the following:
% global BTFBAS;
% global BTFBAS_time;
% global KPFBAS;
% global KPFBAS_time;

global VS_OX;
global mu_vector;
global SS_X_KP;
global SS_X_BT;
global SS_O;
global SS_Glc;
global SS_Dex;
global SS_Ac;
global SS_X_KP_all;
global SS_X_BT_all;

BTFBAS = [];
BTFBAS_time = [];
KPFBAS = []; 
KPFBAS_time =[];
VS_OX = [];
mu_vector = [];
SS_X_KP = [];
SS_X_BT = [];
SS_O = [];
SS_Glc = [];
SS_Dex = [];
SS_Ac = [];



%% -------------------- FOR HYSTERESIS RUN THIS SECTION ------------------- %%%


% tic
% tspan = [0:0.005:150]; 
% community.Sfeed = [6 0.06 1.7 0 0];
% [V, X_KP, X_BT, S_Glc, S_Dex, S_Ac, S_Mal, O] = dmmm2_BtKp_two_CSTR_hysteresis(community, tspan, initialConditions);
% X_KP_cellspermL = X_KP*(1.18*10^12)/1000;
% X_BT_cellspermL = X_BT*(2.11*10^12)/1000; 
% 
% SS_X_KP = [SS_X_KP X_KP(end)];
% SS_X_BT = [SS_X_BT X_BT(end)];
% SS_O = [SS_O O(end)];
% SS_Glc = [SS_Glc S_Glc(end)];
% SS_Dex = [SS_Dex S_Dex(end)];
% SS_Ac = [SS_Ac S_Ac(end)];
% 
% time=toc;
% disp(datestr(datenum(0,0,0,0,0,time),'HH:MM:SS'))



%% --------------------- FOR STATE SWITCH RUN THIS SECTION ---------------- %%

% Note for 3.25 mM glucose run the simulation longer at 70 hours to ensure
% the system reaches steady state. G_flow=[3.25];

tic
G_flow = [0.25 0.5 0.75 1 1.25 1.5 1.75 2 2.25 2.5 2.75 3 3.25 3.5 3.75 4 4.25 4.5 4.75 5 5.25 5.5 5.75 6];
tspan = [0:0.005:50];   
for counter = 1:length(G_flow)
    counter
    value = G_flow(counter);
    value
    community.Sfeed = [value 0.06 1.7 0 0];
    [V, X_KP, X_BT, S_Glc, S_Dex, S_Ac, S_Mal, O] = dmmm2_BtKp_two_CSTR_ss(community, tspan, initialConditions);

    SS_X_KP = [SS_X_KP X_KP(end)];
    SS_X_BT = [SS_X_BT X_BT(end)];
    SS_O = [SS_O O(end)];
    SS_Glc = [SS_Glc S_Glc(end)];
    SS_Dex = [SS_Dex S_Dex(end)];
    SS_Ac = [SS_Ac S_Ac(end)];
    

end

toc;
disp(datestr(datenum(0,0,0,0,0,time),'HH:MM:SS'))



